function [TransMatrix, cholCov] = Tintseig(V ,E, d, QVit, h)
% Returns the discrete transition matrix TransMatrix and Choleski factor cholCov of the
% discrete transition error variance matrix for the integral over the interval h of a 
% continuous time system using a diagonalized state representation specified by the
% function arguments.

% disp('Release version of Tintseig')
eps=0.00001;
[m,n] = size(QVit);
edh=exp(d*h);
TransMatrix=real( V*diag(edh)/V );
Eh=E*h;
E1=(edh*edh'-1)./Eh;
E2=1./(1-Eh/2+Eh.^2/12);
E3=(abs(Eh)>eps);
E4=E1.*E3+E2.*(1-E3);
[Fa,p]=chol(E4);
F=zeros(n,n);
if p>0 % ensure that F'F and E agree over non-zero elements
    sub1=[1:(p-1)];
    sub2=[p:n];
    Fb=Fa'\E4(sub1,sub2);
    F(sub1,sub1)=Fa;
    F(sub1,sub2)=Fb;
else
    F=Fa;
end
R=zeros(m*n,n);
sub=[1:m];
mones=ones(m,1);
for i=1:n
    R(sub,:)=QVit.*(mones*F(i,:));
    sub=sub+m;
end
[null,CcholCov]=qr(R,0); 
CcholCov=sqrt(h)*CcholCov*V';
[null,cholCov]=qr([real(CcholCov);imag(CcholCov)],0);
return
