function [mcov,lvec,dem]=VCZARCov(Phi,SigPhi,kappa,laglim,lagnum)
% Generates the multivariate covariance function of a natural VCZAR model
% with coefficients Phi, SigPhi and parameter kappa
% at the number lagnum of distinct lags up to a specified lag limit laglim
% A state formulation is used to form the state covariance function
% and the model covariances extracted
% If the model is not stable the function fails and the state eigenvalues
% are returned in dem

% disp('Release version of function VCZARCov')
h=laglim/lagnum;
lvec=h*[0:lagnum];
[m,null,p]=size(Phi);
mcov=zeros(m,m,(lagnum+1));
[V,T,subm,dem]=VCZARCovVT(Phi,SigPhi,kappa);
mcov(:,:,1)=V(subm,subm); % extract series variance matrix
Th=expm(h*T);
for k=2:(lagnum+1)
    V=Th*V; % apply lag step transition
    mcov(:,:,k)=V(subm,subm); % extract series covariance matrix
end
return



