function [V,T,subm,dem]=VCZARCovVT(Phi,SigPhi,kappa)
% Generates the state variance and transition matrix T of a natural VCZAR model
% with coefficients Phi, SigPhi and parameter kappa.
% The state covariance matrix at a positive lag r is given by C=expm(r*T)*V.
% The series covariance is then C(subm,subm).
% If the model is not stable the function fails and the state eigenvalues
% are returned in dem

% disp('Release version of function VCZARCov')
[m,null,p]=size(Phi);
eqlim=0.01; % determines eigen method (small value) or Pade method (large) for update
eqlim=100.0;
qpade=6; % order of Pade approximations
%
% transform to predictive form to set up state transition
[Xi,SigXi] = VCZARnatural2predictive(Phi,SigPhi, m, p, kappa);
%% set up state structures
predictive=1; % Option of two equivalent methods using natural or predictive model form
dim = p*m; % state dimension
subm=[1:m];
if predictive==1
    % Define the transition matrix direct form:
    % Left factor
    L = eye(dim) + [zeros(m,dim); [eye(dim-m), zeros(dim-m,m)]];
    % Right factor
    R = [reshape(Xi,m,m*p); kappa*([eye(dim-m), zeros(dim-m,m)]-[zeros(dim-m,m), eye(dim-m)])];
    T=L\R; % state transition matrix
    Gt = zeros(m,dim);
    Gt(1:m, 1:m) = chol(SigXi);
    Gs = Gt*inv(L)'; % right factor of state space disturbance covariance
else
    % Define the transition matrix natural form:
    dimm=dim+m;
    % Left factor
    L = eye(dimm) + [zeros(m,dimm); [eye(dim), zeros(dim,m)]];
    L(1:m,(m+1:dimm))=-reshape(Phi,m,dim);
    % Right factor
    R = -eye(dimm) + [zeros(m,dimm); [eye(dim), zeros(dim,m)]];
    R(1:m,(m+1:dimm))=reshape(Phi,m,dim);
    Tx=L\R; 
    Gtx = zeros(m,dimm);
    Gtx(1:m, 1:m) = chol(SigPhi);
    Gsx = Gtx*inv(L)'; % right factor of state space disturbance covariance
    T=kappa*Tx(1:dim,1:dim); % state transition matrix
    Gs=Gsx(1:m,1:dim);
end

% eigen analysis for possible use in updating
stab=0;
[Vem,Eem,dem,GsVit,stab,eqcrit]=eigsort(T,Gs);
%% Check for stability of the model
if stab==1 % model unstable
    %     disp('model unstable')
    return
end
if eqcrit<eqlim
    eqind=1;
    [pcoef,qfactor,qrank]=pnformT(qpade);
    [TscPow,GTtscPow,scale]=powscalT(T,Gs,qpade);
else
    eqind=0;
end

%% Initialise right Choleski factor P of state covariance matrix
%% by integrating over a very long period
initsc=1.e6;
hin=(1/kappa)*initsc; % initialisation period
if eqind==1
    %    disp('Pade method')
    [Tdisc, cholP] = ...
        Tintspade(TscPow,GTtscPow,scale,Gs,hin,qpade,pcoef,qfactor,qrank); % long period transition
else
    %      disp('eigen method')
    [Tdisc, cholP] = Tintseig(Vem,Eem,dem,GsVit,hin); % long period transition
end
V=cholP'*cholP;
return



