function [mspec,fvec,wspec,gvec]=VCZARSpecf(Phi,SigPhi,kappa,flim,fnum)
% forms the multivariate spectrum mspec of a natural VCZAR model
% over an equal spacing of fnum un-warped frequencies fvec up to the 
% frequency flim returning also the warped spectrum wspec at the 
% corresponding warped frequencies gvec

% disp('Release version of function VCZARSpecf')
%
[m,null,p]=size(Phi);
fr=[0:fnum]*flim/fnum; % unwarped frequencies
SigPhiNoise=SigPhi/(2*kappa);
gr=atan(2*pi*fr/kappa)/pi; % warped frequencies
J=(2/kappa)*(cos(pi*gr)).^2; % Jacobian
temspec=zeros(m,m,fnum+1);
% form warped spectrum
for j=1:(fnum+1)
    csum=eye(m);
    for k=1:p
        B=exp(2*pi*i*gr(j));
        csum=csum-Phi(:,:,k)*B^k;
    end
    csum=inv(csum);
    csum=csum*SigPhiNoise*csum';
    temspec(:,:,j)=csum;
    wspec(:,:,j)=csum;
    mspec(:,:,j)=csum*J(j); %transform to unwarped spectrum
end
fvec=fr;
gvec=gr;
return



