function [mspec,fvec,wspec,gvec]=VCZARSpecg(Phi,SigPhi,kappa,flim,fnum)
% forms the multivariate spectrum mspec of a natural VCZAR model
% over a range of fnum un-warped frequencies fvec up to the frequency 
% flim corresponding to equally spaced frequencies gvec of the warped 
% spectrum wspec

% disp('Release version of function VCZARSpecg')
%
[m,null,p]=size(Phi);
ng=fnum;
SigPhiNoise=SigPhi/(2*kappa);
gr=0.5*[0:ng]/ng;
flind=floor(2*ng*atan(2*pi*flim/kappa)/pi)+1;
fr=(kappa/(2*pi))*tan(pi*gr(1:flind));
J=(2/kappa)*(cos(pi*gr)).^2; % Jacobian
temspec=zeros(m,m,ng+1);
for j=1:(ng+1)
    csum=eye(m);
    for k=1:p
        B=exp(2*pi*i*gr(j));
        csum=csum-Phi(:,:,k)*B^k;
    end
    csum=inv(csum);
    csum=csum*SigPhiNoise*csum';
    temspec(:,:,j)=csum;
    wspec(:,:,j)=csum;
    if j<=flind
        mspec(:,:,j)=csum*J(j);
    end
end

fvec=fr;
gvec=gr;

return



