function [stanperr, predsdev, dev, stab, DSig, NSig, xregcoeff, xregvarmx] =...
    VCZARlikfun_reml(yRef,m,p,kappa,obsnoise,xreg,xregnum,parvec,parind,struc)
% Uses State Space formulation of the Continuous Vector ZAR model to
% calculate likelihood quantities
% Regressors are included as extra state variables for reml computation
% Final time regression coefficients and conditional variances returned

% disp('Release version of VCZARlikfun_reml')
stab=0;% Initialise to indicate model stable
NSig=0; % Initialise to indicate observation noise variance VCZARSig positive
DSig=0; % Initialise to indicate disturbance noise variance VCZARSig positive

[Nobs, is3] = size(yRef);      % number of observed values
tspan=yRef(Nobs,3)-yRef(1,3); % overall time span
hamo=tspan*1.e-12; % successive times closer than this are taken to be the same.
eqlim=0.01; % determines eigen method (small value) or Pade method (large) for update
eqlim=100.0;
qpade=6; % order of Pade approximations

%% initialize output quantities
stanperr = zeros(Nobs,1);
predsdev = zeros(Nobs,1);
dev = 0;

xregtot=sum(xregnum);
xregcoeff=zeros(xregtot,1);
xregvarmx=zeros(xregtot);

%% map to model parameters
xregorigins=cumsum([0,xregnum(1:(m-1))]); % for use in locating coefficients in xregcoeff

[VCZARcoeff,VCZARSig,NoiseSig,Strucoeff]=parvec2true_reml(parvec,m,p,obsnoise,struc);

if obsnoise==1
    if min(NoiseSig)<0
        NSig=1;
    else
        NoiseSD=sqrt(NoiseSig);
    end
elseif obsnoise==2
    [NoiseSD,pInd]=chol(NoiseSig);
    if pInd>0
        NSig=1;
    end
end

% map the structural parameters to standard form 

Sig=VCZARSig;
coeff=VCZARcoeff;
if struc>0
    Sig=(Strucoeff\Sig)/Strucoeff';
end
if struc==2
    for k=1:p
        coeff(:,:,k)=Strucoeff\coeff(:,:,k);
    end
end
if parind==1
    [Xi,SigXi,M] = VCZARnatural2predictive(coeff,Sig, m, p, kappa); 
else
    Xi=coeff;
    SigXi=Sig; % superceded by more direct derivation below
end
%% set up state structures
dim = p*m; % state dimension
dimr=dim+xregtot; % for added xregcoeff states
dimx = dimr; % length of householder vector
if obsnoise>0
    dimx=dimx+ m; % augmented if observation noise
end
% Define the transition matrix direct form:
% Left factor
L = eye(dim) + [zeros(m,dim); [eye(dim-m), zeros(dim-m,m)]];
% Right factor
R = [reshape(Xi,m,m*p); kappa*([eye(dim-m), zeros(dim-m,m)]-[zeros(dim-m,m), eye(dim-m)])];

T=L\R; % state transition matrix

%% right factor of state space disturbance covariance
Gt = zeros(m,dim);
if struc==0
    [R,pInd] = chol(VCZARSig); 
    if pInd>0
        DSig=1;
    end
else 
    dDSig=diag(VCZARSig);
    if min(dDSig)<0
        DSig=1;
    end
    R= diag(sqrt(dDSig));
    R=R/(Strucoeff');
end
if (NSig+DSig)>0 % model noise coefficients invalid
    return
end
if parind==1
    R=R/(M');
end
Gt(1:m, 1:m)=R;
Gs = Gt*inv(L)'; % right factor of state space disturbance covariance

% eigen analysis for possible use in updating
[Vem,Eem,dem,GsVit,stab,eqcrit]=eigsort(T,Gs);
%% Check for stability of the model
if stab==1 % model unstable
    return
end
% eqcrit
if eqcrit<eqlim
    eqind=1;
    [pcoef,qfactor,qrank]=pnformT(qpade);
    [TscPow,GTtscPow,scale]=powscalT(T,Gs,qpade);
    % these could be generated outside of VCZARlikfun
else
    eqind=0;
end

%% Initialise right Choleski factor P of state covariance matrix
initsc=1.e6;
hin=(1/kappa)*initsc; % initialisation period 
% Householder vectors and matrix resultant
F=zeros(dimx,1);
Lh=zeros(dimx,1);
stsub=[1:dim]; % to address stochastic state subvectors
xrsub=[dim+1:dimr]; % to address xreg state subvectors
stxrsub=[1:dimr]; % to address both
if obsnoise>0
    obsub=[dimr+1:dimx]; % to address obs. noise subvectors
end
mzero=zeros(m,1);
R=zeros(dimx+1,dimx);
Sest = zeros(dimr,1); % initial state estimate including regressors
SestA = zeros(dimx,1); % augmented state vector including obs noise
cholP = zeros(dimr); % initialize for distant past including regressors
cholPAzero = zeros(dimx); % full augmented state variance matrix initalise
count=0;
for i=1:m
    y=yRef(find(yRef(:,1)==i),4);
    j=xregnum(i);
    if j>0
        for k=1:j
            count=count+1;
            xregscale(count)=norm(y)/norm(xreg{i}(:,k));
        end
    end
end
Large=1.e7;
% xregscale to initialize state variance ignorance for regressors
cholP(xrsub,xrsub)=Large*diag(xregscale); % xregscale should be external
TdiscAugzero= zeros(dimr); % xreg augmented transition matrix initialise

% Filtering loop
tpast = yRef(1,3)-hin;
% disp('starting likfun time loop')
for seq = 1:Nobs
    sernum=yRef(seq,1);
    serindex=yRef(seq,2);
    tnow = yRef(seq,3);
    seriesval=yRef(seq,4);
    %% adjust for regressors
    jreg=xregnum(sernum);
    if tnow > tpast + hamo % apply transition update
        h = tnow - tpast;
        if eqind==1
            [Tdisc, cholT] = ...
                Tintspade(TscPow,GTtscPow,scale,Gs,h,qpade,pcoef,qfactor,qrank);
        else 
            [Tdisc, cholT] = Tintseig(Vem,Eem,dem, GsVit, h);
        end
        %         Augment transition matrix and disturbance variance for xregcoeff
        TdiscAug=TdiscAugzero;
        TdiscAug(stsub,stsub)=Tdisc;
        TdiscAug(xrsub,xrsub)=eye(xregtot);
        Sest = TdiscAug*Sest;  % updated state estimate including xregs
        %       In construction of A cholP is size dimr^2 in first cycle but is
        %       returned as size dimx by dimr in subsequent cycles. But it is
        %       reduced again to dimr^2 after application of qr to A, when it holds
        %       the updated cholesky factor of the state variance mx.
        [rnk,null]=size(cholT);
        xrzero=zeros(rnk,xregtot); % to fill variance matrix
        A = [cholP*TdiscAug'; [cholT, xrzero] ];
        [null, cholP] = qr(A, 0);
        %       augment state and variance to allow for option of obs noise        
        SestA(stxrsub)=Sest;  
        if obsnoise>0
            SestA(obsub)=mzero;
        end
        cholPA=cholPAzero;
        cholPA(stxrsub,stxrsub)=cholP;
        if obsnoise==1
            cholPA(obsub,obsub)=diag(NoiseSD);
        elseif obsnoise==2
            cholPA(obsub,obsub)=NoiseSD;
        end
        %       transition update completed
    end 
    tpast = tnow;
    %   prediction step follows
    ypred = SestA(sernum);     % prediction of the series observed at this time
    if jreg>0
        for j=1:jreg
            xregval=xreg{sernum}(serindex,j);
            ypred=ypred+SestA(dim+xregorigins(sernum)+j)*xregval;
        end
    end
    if obsnoise>0
        ypred=ypred+SestA(dimr+sernum);
    end
    F = cholPA(:,sernum);  % prediction factor for Householder transform
    if jreg>0
        for j=1:jreg
            F=F+cholPA(:,dim+xregorigins(sernum)+j)*xreg{sernum}(serindex,j);
        end
    end
    if obsnoise>0
        F = F + cholPA(:,dimr+sernum);
    end
    %   state update from prediction follows
    R = cholPA;
    [Lh,cholC] = hhrv(F);          % householder reflection vector and scalar
    % cholC'*cholC is prediction error variance
    R = R-2*Lh*(Lh'*R);           % application of reflection to R
    K = R(1,:)';                  % Kalman gain as applied to standardised errors
    R(1,:) = 0;
    cholPA=R; % right factor of updated state variance
    cholP=cholPA(:,stxrsub);  % retained for use after time increment
    signC = 2*(cholC>0)-1;
    prederr = seriesval-ypred ;    % prediction error
    serr = prederr/cholC; 
    stanperr(seq) = serr*signC;   % standardized error series
    predsdev(seq) = cholC*signC;
    SestA = SestA + K*serr;         % updated state estimates
    Sest = SestA(stxrsub); % retained for use after time increment
end
% completed likfun time loop
% extract regression coefficients estimated as states
xregcoeff=Sest(xrsub);
xregvarmx=cholP(:,xrsub)'*cholP(:,xrsub);
% locate the prediction error terms nullified by state estimation of regression coefficients
posind=ones(Nobs,1);
for j=1:m
    loc=find(yRef(:,1)==j);
    posind(loc(1:xregnum(j)))=0;
end
suberr=find(posind==1);
% stanperr
% predsdev
stanperr=stanperr(suberr);
predsdev=predsdev(suberr);
logprod = sum(log(predsdev));
sumstanperr=sum(stanperr.^2);
dev = sumstanperr+2*logprod; % edited for Large variance errors
% disp('leaving likfun')
return


